#the-container {
  margin: 1em 100px;
  text-align: center;
  background-color: white;
}
#body {
  text-align: justify;
}
div.content {
  padding: 0 7px;
}

/*
14d2f2ca3732551d1585e7590e60b82492f3
^^ A rather nice chroma hash
*/
body {
  background-color: #555;
}

thead, tfoot {
  /* FTR all table columns should be sortable hence the colour to indicate clickableness. */
  color: navy;
  font-size: small;
  font-weight: bold !important;
}
thead th {
  border-bottom: solid 1px #777;
}
tfoot td {
  border-top: solid 1px #777;
}

.listing tbody tr:nth-child(even) {
  background-color: #f7f7f7;
}
.listing tbody tr:hover {
  background-color: #fefeaa;
}

.chroma-hash {
  font-family: monospace;
  font-size:   1em;
  font-style: normal;
}
div.chroma-hash {
  float: right;
}

.time-since {
  font-style: italic;
}
.author, .head {
  font-weight: bold;
  font-family: "Trebuchet MS", "Lucida Grande", serif;
}

.file-name, .file-mode {
  font-family: monospace;
}
.action-list {
  font-size: smaller;
}

.path {
  border-bottom: solid 1px #ddd;
  padding: 3px 0;
  font-weight: bold;
}


/* header */
#page-header {
  height: 25px;
  padding: 8px;
  font-size: 1.5em;
  font-weight: bold;
  border-bottom: solid 1px #777;
  text-align: justify;
}

img.logo {
  float: right;
  border-width: 0px;
  margin-top: 4px;
}

/* footer */
#page-footer {
  height: 20px;
  padding: 8px;
  margin-top: 10px;
  font-style: italic;
  background-color: #d9d8d1;
  border-top: solid 1px #777;
}

/* actions include */
.actions {
  padding-bottom: 4px 0;
  font-style: italic;
  border-bottom: solid 1px #777;
  margin-bottom: 10px;
  padding: 5px 7px;
}

/* pager include */
.pager {
  text-align: center;
}

/* search include */
#page-search {
  text-align: right;
  float: right;
  font-size: 0.7em;
  padding-right: 20px;
}

/* shortlog include */
table.shortlog tbody tr {
  padding: 1px 0px;
}
table.shortlog tbody td {
  padding: 0 1px;
}

/* /commit page */
.commit-message {
  font-family: monospace;
  font-size: 1.2em;
}
div.commit-message {
  margin-bottom: 10px;
}
pre.commit-message {
  border-top:  solid 2px red;
  border-bottom:  solid 2px green;
  padding: 5px;
}
.commit-info dt {
  font-weight: bold;
}
.commit-info dd {
  font-family: monospace;
}

/* /heads */
.heads .head {
  font-weight: bold;
}
.heads .current {
  text-decoration: underline;
}
table.heads {
  width: 75%;
}

/* /blame */
#blame pre, #blame tt {
  margin: 0;
  font-size: 0.9em;
}
#blame .commit-info {
}
#blame .lineno {
  text-align: right;
  padding: 0 8px;
}
#blame a {
  text-decoration: none;
}
#blame {
    overflow-x: scroll;
}
#blame tr.alt {
  background-color: #f7f7f7;
}
#blame tbody tr:hover {
  background-color: #fefeaa;
}
#blame td {
  padding: 0 0.2em;
}
#blame td.lineno {
  background-color: #eee;
}
#blame td.date, #blame td.author, #blame td.commit-info {
}
#blame .data {
  padding-left: 5px;
}

/* /blob */
pre.blob {
  background-color: #333;
  color: #ddd;
  border-left: solid 3px #c33;
  padding: 5px;
  padding-left: 15px;
  margin: 10px 15px;
}

/* /blobdiff etc */

.diff-head, .diff-index {
  font-family: monospace;
}
.diff-head {
  border-top: solid 1px red;
}
.diff-index {
  border-bottom: solid 1px green;
}

/* /commitdiff */
.diff-patch {
  font-size: 0.8em;
}

/* /log */
#log .entry {
  border: solid 1px grey;
  margin: 5px 0;
  padding: 5px;
}
#log .meta {
  border-top: dotted 1px #ddd;
  color: #311;
}
#log table.summary {
  width: 33%;
  font-size: 0.9em;
}
#log .message {
  font-family: monospace;
  font-size: 1.15em;
  /* XXX Need to enforce find the CSS switch to force the chroma-hash below the message box */
}
#log .age {
  float: right;
  font-size: 0.85em;
  font-style: italic;
}

/* /summary */
#stats {
  float: right;
}

/* /tree */
table.tree {
  width: 65%;
}

/* Formerly of gitweb.css */

span.refs span {
  padding: 0px 4px;
  font-size: 80%;
  /* XXX needs more families */
  font-family: Verdana;
  letter-spacing: -1px;
  border: 1px solid;
  background-color: #ffaaff;
  border-color: #ffccff #ff00ee #ff00ee #ffccff;
}

span.refs span.ref {
  background-color: #aaaaff;
  border-color: #ccccff #0033cc #0033cc #ccccff;
}

span.refs span.tag {
  background-color: #ffffaa;
  border-color: #ffffcc #ffee00 #ffee00 #ffffcc;
}

span.refs span.head {
  background-color: #aaffaa;
  border-color: #ccffcc #00cc33 #00cc33 #ccffcc;
}
span.refs a {
  text-decoration: none;
}

a.rss_logo {
  float: right;
  padding: 3px 0px;
  width: 35px;
  line-height: 10px;
  border: 1px solid;
  border-color: #fcc7a5 #7d3302 #3e1a01 #ff954e;
  color: #ffffff;
  background-color: #ff6600;
  font-weight: bold;
  font-family: sans-serif;
  font-size: 70%;
  font-style: normal;
  text-align: center;
  text-decoration: none;
}

a.rss_logo:hover {
  background-color: #ee5500;
}
